<?php

namespace Bixo\Workflow\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Workflow\Http\Requests\WorkflowRequest;
use Bixo\Workflow\Interfaces\WorkflowRepositoryInterface;
use Bixo\Workflow\Models\Workflow;
use Bixo\Workflow\Forms\Workflow as Form;

/**
 * APIController  class for workflow.
 */
class WorkflowAPIController extends BaseController
{

    /**
     * Initialize workflow resource controller.
     *
     * @param type WorkflowRepositoryInterface $workflow
     *
     * @return null
     */
    public function __construct(WorkflowRepositoryInterface $workflow)
    {
        parent::__construct();
        $this->repository = $workflow;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Workflow\Repositories\Criteria\WorkflowResourceCriteria::class);
    }

    /**
     * Display a list of workflow.
     *
     * @return Response
     */
    public function index(WorkflowRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Workflow\Repositories\Presenter\WorkflowPresenter::class)
            ->paginate();
    }

    /**
     * Display workflow.
     *
     * @param Request $request
     * @param Model   $workflow
     *
     * @return Response
     */
    public function show(WorkflowRequest $request, Workflow $workflow)
    {
        return $workflow->setPresenter(\Bixo\Workflow\Repositories\Presenter\WorkflowListPresenter::class);
        ;
    }

    /**
     * Create new workflow.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(WorkflowRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('workflow::workflow.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('workflow/workflow/' . $workflow->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('workflow/workflow');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the workflow.
     *
     * @param Request $request
     * @param Model   $workflow
     *
     * @return Response
     */
    public function update(WorkflowRequest $request, Workflow $workflow)
    {
        try {
            $data = $request->all();

            $workflow->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('workflow::workflow.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('workflow/workflow/' . $workflow->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('workflow/workflow/' . $workflow->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the workflow.
     *
     * @param Model   $workflow
     *
     * @return Response
     */
    public function destroy(WorkflowRequest $request, Workflow $workflow)
    {
        try {
            $workflow->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('workflow::workflow.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('workflow/workflow/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('workflow/workflow/' . $workflow->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
