<?php

namespace Bixo\Workflow\Policies;

use Litepie\User\Contracts\UserPolicy;
use Bixo\Workflow\Models\Workflow;

class WorkflowPolicy
{

    /**
     * Determine if the given user can view the workflow.
     *
     * @param UserPolicy $user
     * @param Workflow $workflow
     *
     * @return bool
     */
    public function view(UserPolicy $user, Workflow $workflow)
    {
        if ($user->canDo('workflow.workflow.view') && $user->isAdmin()) {
            return true;
        }

        return $workflow->user_id == user_id() && $workflow->user_type == user_type();
    }

    /**
     * Determine if the given user can create a workflow.
     *
     * @param UserPolicy $user
     * @param Workflow $workflow
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('workflow.workflow.create');
    }

    /**
     * Determine if the given user can update the given workflow.
     *
     * @param UserPolicy $user
     * @param Workflow $workflow
     *
     * @return bool
     */
    public function update(UserPolicy $user, Workflow $workflow)
    {
        if ($user->canDo('workflow.workflow.edit') && $user->isAdmin()) {
            return true;
        }

        return $workflow->user_id == user_id() && $workflow->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given workflow.
     *
     * @param UserPolicy $user
     * @param Workflow $workflow
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Workflow $workflow)
    {
        return $workflow->user_id == user_id() && $workflow->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given workflow.
     *
     * @param UserPolicy $user
     * @param Workflow $workflow
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Workflow $workflow)
    {
        if ($user->canDo('workflow.workflow.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given workflow.
     *
     * @param UserPolicy $user
     * @param Workflow $workflow
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Workflow $workflow)
    {
        if ($user->canDo('workflow.workflow.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
